"use strict";

import * as GlobalData from "../Global/GlobalData.js";
import * as GameField  from "../Global/GameField.js";
import * as View       from "./View.js";

let explosionNumber = GlobalData.EXPLOSION_NUMBER_MIN;

export function calculateTextXcoordinate(text)
{
	return ((GlobalData.CANVAS_WIDTH - View.context.measureText(text).width) / 2);
}

export function getRandomInteger(min, max) // Both values are [inclusive].
{
	return Math.floor(Math.random() * (max - min + 1) ) + min;
}

export function getColor(tileNumber)
{
	let color;
	if (tileNumber >= GlobalData.EXPLOSION_NUMBER_MIN)
	{
		color = GlobalData.enumColor.BLACK;
	}
	else
	{
		switch (tileNumber)
		{
			case GlobalData.enumTileNumber.RED      : color = GlobalData.enumColor.RED    ; break;
			case GlobalData.enumTileNumber.GREEN    : color = GlobalData.enumColor.GREEN  ; break;
			case GlobalData.enumTileNumber.BLUE     : color = GlobalData.enumColor.BLUE   ; break;
			case GlobalData.enumTileNumber.YELLOW   : color = GlobalData.enumColor.YELLOW ; break;
			case GlobalData.enumTileNumber.NO_TILE  : color = GlobalData.enumColor.BLACK  ; break;
			default                                 : color = GlobalData.enumColor.UNDEFINED;
		}
	}
	return color;
}

export function getTileX(col)
{
	return (col * GlobalData.TILE_SIZE) + GlobalData.OFFSET_GAME_FIELD_LEFT;
}

export function getTileY(row)
{
	return (row * GlobalData.TILE_SIZE) + GlobalData.OFFSET_GAME_FIELD_TOP;
}

export function isMouseClickWithinGameField(x,y)
{
	return x > GlobalData.X_GAMEFIELD_TOP_LEFT     &&
         x < GlobalData.X_GAMEFIELD_BOTTOM_RIGHT &&
         y > GlobalData.Y_GAMEFIELD_TOP_LEFT     &&
         y < GlobalData.Y_GAMEFIELD_BOTTOM_RIGHT;
}

export function createTileLocation(row, col)
{
	return `${row}${GlobalData.UNDERSCORE}${col}`;
}

export function getRowAndColArray(tileKey)
{
	const rowAndColAsString = tileKey.split(GlobalData.UNDERSCORE);

	const rowTarget = parseInt(rowAndColAsString[0]);
	const colTarget = parseInt(rowAndColAsString[1]);

	const rowAndCol = [];
	rowAndCol.push(rowTarget);
	rowAndCol.push(colTarget);

	return rowAndCol;
}

export function areThereTileClustersLeft()
{
	let tileClusterFound = false;
	for (let col = 0 ; col < GlobalData.COL_COUNT ; col++)
	{
		if (tileClusterFound)
		{
			break;
		}

		for (let row = GlobalData.ROW_MAX ; row > GlobalData.ROW_MIN ; row--)
		{
			let tile = GameField.data[row][col];
			if (isEmptyTile(tile))
			{
				break;
			}

			let tileAbove = row > GlobalData.ROW_MIN ? GameField.data[row - 1][col] : GlobalData.enumTileNumber.NO_TILE;
			let tileRight = col < GlobalData.COL_MAX ? GameField.data[row][col + 1] : GlobalData.enumTileNumber.NO_TILE;

			if (tile === tileAbove || tile === tileRight)
			{
				tileClusterFound = true;
				break;
			}
		}
	}
	return tileClusterFound;
}

export function countRemainingTiles()
{
	let tileCount = 0;
	for (let col = 0 ; col < GlobalData.COL_COUNT ; col++)
	{
		for (let row = GlobalData.ROW_MAX; row > GlobalData.ROW_MIN; row--)
		{
			let tile = GameField.data[row][col];
			if (isValidTile(tile))
			{
				tileCount++;
			}
			else
			{
				break;
			}
		}
	}
	return tileCount;
}

export function isValidTile(tile)
{
	return (tile !== undefined && tile <= GlobalData.TILE_NUMBER_MAX);
}

export function isValidTileOrExplosion(tile)
{
	return ((tile !== undefined && tile !== GlobalData.enumTileNumber.NO_TILE) || tile >= GlobalData.EXPLOSION_NUMBER_MIN);
}

export function isExplosion(tile)
{
	return (tile !== undefined && tile >= GlobalData.EXPLOSION_NUMBER_MIN);
}

export function isEmptyTile(tile)
{
	return (tile !== undefined && tile === GlobalData.enumTileNumber.NO_TILE);
}

export function isEmptyTileOrExplosion(tile)
{
	return (isEmptyTile(tile) || tile >= GlobalData.EXPLOSION_NUMBER_MIN);
}

// export function calculateRelativeDistance(row1, col1, row2, col2)
// {
// 	return Math.abs(row1 - row2) + Math.abs(col1 - col2);
// }

export function isLocationWithinBounds(point, pointStart, pointEnd)
{
	const b1 = (point.X > pointStart.X);
	const b2 = (point.X < pointEnd.X);

	const b3 = (point.Y > pointStart.Y);
	const b4 = (point.Y < pointEnd.Y);

	return (b1 && b2 && b3 && b4);
}


export function getNewExplosionNumber()
{
	explosionNumber++;
	if (explosionNumber >= GlobalData.EXPLOSION_NUMBER_MAX)
	{
		explosionNumber = GlobalData.EXPLOSION_NUMBER_MIN;
	}
	return explosionNumber;
}

export function drawOvalStroked(X, Y, width, radius)
{
	View.context.lineWidth = 3.0;
	View.context.strokeStyle = "orange";

	View.context.beginPath();
	View.context.arc(X,Y, radius, 0.5 * Math.PI, 1.5 * Math.PI, false); // Halve cirkel linkerkant.
	View.context.lineTo(X += width, Y -= radius);
	View.context.arc(X, Y + radius, radius, 1.5 * Math.PI, 0.5 * Math.PI, false); // Halve cirkel rechterkant.
	View.context.closePath();
	View.context.stroke();
}
